/* ============================================================================
* History
* =======
* V1_0    26/05/2015       Created      GB
*
* (C) Copyright 2014-2015, Future Technology Devices International Ltd.
* ============================================================================
*
* This source code ("the Software") is provided by Future Technology Devices
* International Limited ("FTDI") subject to the licence terms set out
* http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
* You must read the Licence Terms before downloading or using the Software.
* By installing or using the Software you agree to the Licence Terms. If you
* do not agree to the Licence Terms then do not download or use the Software.
*
* Without prejudice to the Licence Terms, here is a summary of some of the key
* terms of the Licence Terms (and in the event of any conflict between this
* summary and the Licence Terms then the text of the Licence Terms will
* prevail).
*
* The Software is provided "as is".
* There are no warranties (or similar) in relation to the quality of the
* Software. You use it at your own risk.
* The Software should not be used in, or for, any medical device, system or
* appliance. There are exclusions of FTDI liability for certain types of loss
* such as: special loss or damage; incidental loss or damage; indirect or
* consequential loss or damage; loss of income; loss of business; loss of
* profits; loss of revenue; loss of contracts; business interruption; loss of
* the use of money or anticipated savings; loss of information; loss of
* opportunity; loss of goodwill or reputation; and/or loss of, damage to or
* corruption of data.
* There is a monetary cap on FTDI's liability.
* The Software may have subsequently been amended by another user and then
* distributed by that other user ("Adapted Software").  If so that user may
* have additional licence terms that apply to those amendments. However, FTDI
* has no liability in relation to those amendments.
* ============================================================================
*/

// @file                           AN_356_Code_v1_0.ino
// @brief                          This application demonstrates connection of I2C sensors to the VM800P board
//                                 Tested platform version: Arduino 1.0.5 r2 with VM800P 4.3"

/* Arduino standard includes */
#include "SPI.h"
#include "Wire.h"
#include "digit.h"

/* Platform specific includes */
#include "FT_VM800P43_50.h"

#define I2CADDRESS	0x40	// I2C address of INA219 is 0x40 (see INA219 datasheet for configuration)

/* Global object for FT800 Implementation */
FT800IMPL_SPI FTImpl(FT_CS_PIN,FT_PDN_PIN,FT_INT_PIN);

/* Api to bootup ft800, verify FT800 hardware and configure display/audio pins */
/* Returns 0 in case of success and 1 in case of failure */
int16_t BootupConfigure()
{
    uint32_t chipid = 0;
    
    // Configure the display to the WQVGA
    FTImpl.Init(FT_DISPLAY_RESOLUTION);            

    delay(20);

    chipid = FTImpl.Read32(FT_ROM_CHIPID);
	
    /* Identify the chip */
    if(FT800_CHIPID != chipid)
    {
        Serial.print("Error in chip id read ");
	Serial.println(chipid,HEX);
	return 1;
    }
	
    // Set the Display & audio pins
    FTImpl.SetDisplayEnablePin(FT_DISPENABLE_PIN);
    FTImpl.SetAudioEnablePin(FT_AUDIOENABLE_PIN); 
    FTImpl.DisplayOn(); 	
    FTImpl.AudioOn();  		
    return 0;
}







// ####################################################################################################################################
// ############################################ Chart Application #####################################################################
// ####################################################################################################################################

void Chart()
{
    unsigned int VoltageHi = 0;
    unsigned int VoltageLo = 0;
    unsigned int Voltage = 0;
    
    unsigned int CurrentHi = 0;
    unsigned int CurrentLo = 0;
    unsigned int Current = 0;
    
    byte I2C_Status = 0;
    boolean I2C_Error = false; 
    unsigned int TimeoutCounter = 0;
    
    unsigned int ResultBuffer[300];
    unsigned int ChartCurrentValue = 0;
    unsigned int OutputIndex;
    unsigned int ChartIndexX;
    unsigned int InputIndex = 0;

    uint32_t ReadWord = 0;     
    
    // Chart area definitions
    #define ChartWidth     300                        // Chart width
    #define ChartHeight    200                        // Chart height

    #define ChartTL_X      50                         // Chart Top-Left X coordinate 
    #define ChartTL_Y      50                         // Chart Top-Left Y coordinate    
    #define ChartBR_X      (ChartTL_X + ChartWidth)   // Chart Botton-Right X coordinate 
    #define ChartBR_Y      (ChartTL_Y + ChartHeight)  // Chart Botton-Right Y coordinate       
    // Chart division lines    
    #define DivisionSize        50                    // A line every 50 pixels
    #define DivisionXOffset     20                    // Distance of the division labels (e.g. "100") from the left-hand edge of screen
    #define DivisionFontHeight  13                    // font 20, used to center text vertically (defined here but could obtain from font block properties instead)

    #define Division1_Line_LX  ChartTL_X              // Left X coord - X coordinate of the left-hand end of all division lines (equals the X coord of the chart rectangle's left side)
    #define Division1_Line_LY  (ChartBR_Y - DivisionSize)  // Y coordinate of left-hand end of first division line. [divisionsize] above the bottom of the chart
    #define Division1_Line_RX   ChartBR_X             // Right X coord - X coordinate of the right-hand end of all division lines (equals the X coord of the chart rectangle's right side)
    #define Division1_Line_RY   (ChartBR_Y - DivisionSize) // Y coord of right-hand end of line is same as that of the left hand side to make horizontal line
    #define Division1_Label_X   DivisionXOffset       // Label for each division (e.g. "100") is set just in from left-hand side of screen
    #define Division1_Label_Y   (ChartBR_Y - (DivisionFontHeight/2)) // First label (normally "0") is at very bottom of chart. Adjust slightly so that text is centred on the line. 
      
    // Coordinates for Title text
    #define Title_Text_X   (FT_MAX_DISPLAYWIDTH / 2)     // Title text centered ...
    #define Title_Text_Y   20                            // ... near top of screen
    
    // Coordinates for Voltage digital readouts
    #define Voltage_Text_X (FT_MAX_DISPLAYWIDTH - 140)   // "Voltage" meter heading positioned to right-hand side of screen
    #define Voltage_Text_Y (ChartTL_Y)                   // Y coordinate is aligned with the top of the chart

    #define Voltage_Value_X (FT_MAX_DISPLAYWIDTH - 140)  // Voltage digital readout (value) positioned to right-hand side of screen
    #define Voltage_Value_Y (ChartTL_Y + 20)             // Y coordinate is 20 pixels under the Voltage heading
    
    #define Voltage_Units_X (FT_MAX_DISPLAYWIDTH - 80)   // Voltage units ("V") are to right-hand side of the actual readout value
    #define Voltage_Units_Y (ChartTL_Y + 20)             // Y coordinate is aligned with the value itself
 
    // Coordinates for Voltage digital readouts      
    #define Current_Text_X  (FT_MAX_DISPLAYWIDTH - 140)
    #define Current_Text_Y (ChartTL_Y + 60)
    
    #define Current_Value_X (FT_MAX_DISPLAYWIDTH - 140)
    #define Current_Value_Y (ChartTL_Y + 80)
    
    #define Current_Units_X (FT_MAX_DISPLAYWIDTH - 80)
    #define Current_Units_Y (ChartTL_Y + 80)

    // Coordinates for the label "mA" beside the chart Y scale
    #define Chart_Units_X    5
    #define Chart_Units_Y    (ChartTL_Y + (ChartHeight/3))

    // Coordinates for the word "Range" above the range select buttons
    #define Range_Label_X    (FT_MAX_DISPLAYWIDTH - 140)
    #define Range_Label_Y    200    


    // ####################################################################################################################################
    // Variables and Definitions
    // ####################################################################################################################################

    byte CurrentRange = 0;
    #define RANGE_1600 0x00
    #define RANGE_200  0x02
    #define RANGE_800  0x08
   
    // ####################################################################################################################################
    // Load the DS-DIGITS font into the RAM_G
    // ####################################################################################################################################
    
    FTImpl.Writefromflash(FT_RAM_G + 1000, MetricBlock, sizeof(MetricBlock)); // Loading data over SPI from digit.h to address RAM_G + 1000
    FTImpl.Writefromflash(FT_RAM_G + 1000 + sizeof(MetricBlock), FontBmpData, sizeof(FontBmpData));
    FTImpl.DLStart();
    FTImpl.Cmd_SetFont(14,1000);                                       // Allocate to font 14
    FTImpl.BitmapHandle(14);                                           // Set bitmap handle as 14
    FTImpl.BitmapSource(-1252);                                        // Set bitmap source (see metric block in digit.h)
    FTImpl.BitmapLayout(FT_L1,3,25);                                   // Format, stride and height we used for the font  
    FTImpl.BitmapSize(FT_NEAREST,FT_BORDER,FT_BORDER,18,25);           // Bitmap size (font width and height)
    FTImpl.DLEnd();                                                    // End the display list
    FTImpl.Finish();                                                   // Render the display list and wait for the completion of the DL

    // ####################################################################################################################################
    // Clear the screen to black colour
    // ####################################################################################################################################
    
    FTImpl.DLStart();
    FTImpl.ClearColorRGB(0,0,0);                                        // Set the clear color to black
    FTImpl.Clear(1,1,1);                                                // Clear the screen
    FTImpl.DLEnd();                                                     // End the display list
    FTImpl.Finish();                                                    // Render the display list and wait for the completion of the DL

    // ####################################################################################################################################
    // Initialise variables and buffers
    // ####################################################################################################################################

    // Clear the values buffer so that chart begins at 0. The chart is 300 pixels wide and so the array has 300 entries

    for(ChartIndexX = 0; ChartIndexX < 300; ChartIndexX ++)
    {
        ResultBuffer[ChartIndexX] = 0;                                  // Set entire array to 0 
    }

    // ####################################################################################################################################
    // Create the Static parts of the display which will not change throughout the running of the code
    // ####################################################################################################################################

    // =========================================== Create the chart background ============================================================

    FTImpl.Write16(REG_CMD_DL,0);                                       // Make sure next display list instruction is started at 0 offset

    FTImpl.Cmd_Gradient(230, 0, 0xFF0000FF, 230, 272, 0x00FF007F);      // Background is a gradient
    
    // White text for legends
    FTImpl.ColorRGB(255,255,255);
    FTImpl.Cmd_Text(Title_Text_X, Title_Text_Y, 23, FT_OPT_CENTER, "FTDI USB Current and Voltage Monitor");  // Title
    FTImpl.Cmd_Text(Voltage_Text_X, Voltage_Text_Y, 21, 0, "Voltage");  // Label beside voltage readout
    FTImpl.Cmd_Text(Current_Text_X, Current_Text_Y, 21, 0, "Current");  // Label beside current readout
    FTImpl.Cmd_Text(Current_Units_X, Voltage_Units_Y, 21, 0, "V");      // Units for voltage readout      
    FTImpl.Cmd_Text(Current_Units_X, Current_Units_Y, 21, 0, "mA");     // Units for current readout
    FTImpl.Cmd_Text(Chart_Units_X, Chart_Units_Y, 21, 0, "mA");         // Units on Y scale of chart   
    FTImpl.Cmd_Text(Range_Label_X, Range_Label_Y, 21, 0, "Range (mA)"); // Label above range selection buttons
            
    // Prepare to draw the rectangle of the chart area
    FTImpl.LineWidth(1 * 16);                                           // Line width
    FTImpl.Begin(FT_RECTS);                                             // Begin rectangles primitive
    // Draw a dark blue rectangle which will form a narrow outline around the chart area
    FTImpl.ColorRGB(0x20,0x20,0x20);                                    // Set the color to dark blue (matches the Division marks used later)
    FTImpl.Vertex2f(((ChartTL_X-1)*16), ((ChartTL_Y -1)*16));           // Top-left coordinates set for 1 pixel larger than chart area
    FTImpl.Vertex2f(((ChartBR_X+1)*16), ((ChartBR_Y+1)*16));            // Bottom-right coordinates set for 1 pixel larger than chart area
    // Now draw a lighter blue rectangle on top which will be the chart background
    FTImpl.ColorRGB(0x00,0x00,0x7F);                                    // Set the color of the rectangle to blue
    FTImpl.Vertex2f((ChartTL_X*16), (ChartTL_Y*16));	                // Top-left coordinates as defined at top of code
    FTImpl.Vertex2f((ChartBR_X*16), (ChartBR_Y*16));	                // Bottom-right coordinates
    FTImpl.End();                                                       // End rectangle primitive

    // Draw the horizontal lines of the display scale/graticule
    FTImpl.ColorRGB(0x0F,0x0F,0x0F);                                    // Set the color to a dark gray
    FTImpl.Begin(FT_LINES);                                             // Begin lines primitive	
    FTImpl.Vertex2f(ChartTL_X*16, ChartBR_Y*16);	                // Line from left side bottom corner...to
    FTImpl.Vertex2f(ChartBR_X*16, ChartBR_Y*16);	                // ... right side bottom corner. This is lowest scale line.
    FTImpl.Vertex2f(ChartTL_X*16, (ChartBR_Y - DivisionSize)*16);	// Another horizontal line from left to right...
    FTImpl.Vertex2f(ChartBR_X*16, (ChartBR_Y - DivisionSize)*16);	// ... which is [Divisionsize] pixels above the bottom of the chart
    FTImpl.Vertex2f(ChartTL_X*16, (ChartBR_Y - DivisionSize*2)*16);	// Another line at the next Division size up
    FTImpl.Vertex2f(ChartBR_X*16, (ChartBR_Y - DivisionSize*2)*16);	//
    FTImpl.Vertex2f(ChartTL_X*16, (ChartBR_Y - DivisionSize*3)*16);	// Another line at the next Division size up
    FTImpl.Vertex2f(ChartBR_X*16, (ChartBR_Y - DivisionSize*3)*16);	//
    FTImpl.Vertex2f(ChartTL_X*16, (ChartBR_Y - DivisionSize*4)*16);	// And finally this line at the top of the chart
    FTImpl.Vertex2f(ChartBR_X*16, (ChartBR_Y - DivisionSize*4)*16);	// 
    FTImpl.End();                                                       // End rectangle primitive
    
    FTImpl.Finish();                                                    // Render the display list and wait until complete

    // The co-processor has now created the display list for the above items. Now we copy this D/L into RAM_G so that it can be recalled later. 
    uint16_t dloffset_backgnd;                                          // This variable will hold the size of the display list
    dloffset_backgnd =  FTImpl.Read16(REG_CMD_DL);                      // Now that we have rendered the D/L, this register points to the end of the DL
    FTImpl.Cmd_Memcpy(100000L,FT_RAM_DL,dloffset_backgnd);              // Copy amount of data [dloffset_backgnd] from RAM_DL into memory from location 100,000




    // =========================================== Create the scale for 200mA range ============================================================


    FTImpl.Write16(REG_CMD_DL,0);                                       // Make sure next display list instruction is started at 0 offset
    // Print the values on the Y scale. Note that the Y position is shifted slightly so it appears centered on the Divisionmark (by using DivisionFontHeight). 
    FTImpl.ColorRGB(255,255,255);
    FTImpl.Cmd_Text(Division1_Label_X, ChartBR_Y - (DivisionFontHeight/2), 20, 0, "   0");                    // Lower label in line with bottom of chart
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize - (DivisionFontHeight/2)), 20, 0, "  50");   // Next label [Divisionsize] above the previous one    
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*2 - (DivisionFontHeight/2)), 20, 0, " 100"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*3 - (DivisionFontHeight/2)), 20, 0, " 150"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*4 - (DivisionFontHeight/2)), 20, 0, " 200"); // Final label at the top of the scale
    FTImpl.End();                                                       // End rectangle primitive
    FTImpl.Finish();                                                    // Render the display list and wait until complete

    // The co-processor has now created the display list for the above items. Now we copy this D/L into RAM_G so that it can be recalled later. 
    uint16_t dloffset_Scale200;                                         // This variable will hold the size of the display list
    dloffset_Scale200 =  FTImpl.Read16(REG_CMD_DL);                     // Now that we have rendered the D/L, this register points to the end of the DL
    FTImpl.Cmd_Memcpy(150000L,FT_RAM_DL,dloffset_Scale200);             // Copy amount of data [dloffset_Scale200] from RAM_DL into memory from location 150,000



    // =========================================== Create the scale for 800mA range ============================================================


    FTImpl.Write16(REG_CMD_DL,0);                                       // Make sure next display list instruction is started at 0 offset
    // Print the values on the Y scale. Note that the Y position is shifted slightly so it appears centred on the Divisionmark (by using DivisionFontHeight). 
    FTImpl.ColorRGB(255,255,255);
    FTImpl.Cmd_Text(Division1_Label_X, ChartBR_Y - (DivisionFontHeight/2), 20, 0, "   0");                    // Lower label in line with bottom of chart
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize - (DivisionFontHeight/2)), 20, 0, " 200");   // Next label [Divisionsize] above the previous one    
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*2 - (DivisionFontHeight/2)), 20, 0, " 400"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*3 - (DivisionFontHeight/2)), 20, 0, " 600"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*4 - (DivisionFontHeight/2)), 20, 0, " 800"); // Final label at the top of the scale
    FTImpl.End();                                                       // End rectangle primitive
    FTImpl.Finish();                                                    // Render the display list and wait until complete

    // The co-processor has now created the display list for the above items. Now we copy this D/L into RAM_G so that it can be recalled later. 
    uint16_t dloffset_Scale800;                                         // This variable will hold the size of the display list
    dloffset_Scale800 =  FTImpl.Read16(REG_CMD_DL);                     // Now that we have rendered the D/L, this register points to the end of the DL
    FTImpl.Cmd_Memcpy(160000L,FT_RAM_DL,dloffset_Scale800);             // Copy amount of data [dloffset_Scale800] from RAM_DL into memory from location 160,000


    // =========================================== Create the scale for 1600mA range ============================================================


    FTImpl.Write16(REG_CMD_DL,0);                                       // Make sure next display list instruction is started at 0 offset
    // Print the values on the Y scale. Note that the Y position is shifted slightly so it appears centred on the Divisionmark (by using DivisionFontHeight). 
    FTImpl.ColorRGB(255,255,255);
    FTImpl.Cmd_Text(Division1_Label_X, ChartBR_Y - (DivisionFontHeight/2), 20, 0, "   0");                    // Lower label in line with bottom of chart
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize - (DivisionFontHeight/2)), 20, 0, " 400");   // Next label [Divisionsize] above the previous one    
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*2 - (DivisionFontHeight/2)), 20, 0, " 800"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*3 - (DivisionFontHeight/2)), 20, 0, "1200"); //
    FTImpl.Cmd_Text(Division1_Label_X, (ChartBR_Y - DivisionSize*4 - (DivisionFontHeight/2)), 20, 0, "1600"); // Final label at the top of the scale
    FTImpl.End();                                                       // End rectangle primitive
    FTImpl.Finish();                                                    // Render the display list and wait until complete

    // The co-processor has now created the display list for the above items. Now we copy this D/L into RAM_G so that it can be recalled later. 
    uint16_t dloffset_Scale1600;                                        // This variable will hold the size of the display list
    dloffset_Scale1600 =  FTImpl.Read16(REG_CMD_DL);                    // Now that we have rendered the D/L, this register points to the end of the DL
    FTImpl.Cmd_Memcpy(170000L,FT_RAM_DL,dloffset_Scale1600);            // Copy amount of data [dloffset_Scale1600] from RAM_DL into memory from location 170,000










    // #################################################################################################################################
    // Initialise INA219
    // #################################################################################################################################


    // Now use the Arduino 'Wire' library to talk to the INA219 over I2C 
    Wire.beginTransmission(I2CADDRESS);                                 // INA219 has address 0x40 in this example (see INA219 datasheet)
    Wire.write((char)(0x00));                                           // Write the arress of the register which we want to select, 00 is the INA219's config register
    Wire.write((char)(0x0E));                                           // The value 0x0EEF selects 80mV shunt gain, constant conversions
    Wire.write((char)(0xEF));                                           // The INA219 datasheet explains the variety of options
    I2C_Status = Wire.endTransmission(true);                            // Send off these bytes over I2C. The true parameter specifies to finish with an I2C stop
      
    if(I2C_Status != 0)
    {
        FTImpl.DLStart();
        FTImpl.ClearColorRGB(64,64,64);
        FTImpl.Clear(1,1,1);    
        FTImpl.ColorRGB(0xff, 0xff, 0xff);
        FTImpl.Cmd_Text((FT_DISPLAYWIDTH/2), (FT_DISPLAYHEIGHT/2), 27, FT_OPT_CENTER, "I2C Sensor Error");
        FTImpl.DLEnd();                                                 // End the display list
        FTImpl.Finish();
        while(1)
        {
        }
    }
          
    // #################################################################################################################################
    // Main program loop
    // #################################################################################################################################
    
    while(1)
    {
        // #################################################################################################################################
        // Get the voltage and current values
        // #################################################################################################################################
      
        I2C_Error = false;
      
        // --------------------------------------- Get the Voltage value from the INA219's Bus Voltage register -------------------------------
        
        // Select bus voltage register
        Wire.beginTransmission(I2CADDRESS);                // I2C Address of INA219 in this example is 0x40
        Wire.write((char)(0x02));                          // First byte sent selects the register in the INA219 which we want to read, 02 is bus voltage 
        Wire.endTransmission(true);                        // Send this command over I2C
         
        Wire.requestFrom(I2CADDRESS, 2, true);             // Request 2 bytes from the INA219 and release the bus with a stop
        
        TimeoutCounter = 0;
        while((Wire.available() < 2) && (TimeoutCounter < 1000))     // wait for 2 bytes back, timeout in approx 1 sec
        {
            TimeoutCounter ++;
            delay(1);
        }   
            
        VoltageHi = (Wire.read());			   // Read high byte of the bus voltage measurement
        VoltageLo = (Wire.read());                         // Read low byte of the bus voltage measurement
     
        // High byte has   --- B11 B10 B09 B08 B07 B06 B05
        // Low byte has    B04 B03 B02 B01 B00 --- --- ---     where Bxx indicate bits of bus voltage measurement, --- indicates don't care in this application
     
        VoltageHi = ((VoltageHi & 0x7F) << 8);             // Mask off upper bit of high byte       --- B11 B10 B09 B08 B07 B06 B05 --- --- --- --- --- --- --- --- 
        VoltageLo = (VoltageLo & 0xF8);                    // Mask off lower 3 bits of low byte     --- --- --- --- --- --- --- --- B04 B03 B02 B01 B00 --- --- ---
        //Voltage = ((VoltageHi + VoltageLo) /8) * 4;      // Combine high and low bytes,           --- --- --- --- B11 B10 B09 B08 B07 B06 B05 B04 B03 B02 B01 B00
                                                           // shift down 3 places so it is starting at bit 0,
                                                           // and multiply by 4 since 4mV per binary value.
        Voltage = ((VoltageHi + VoltageLo) /2);            // Optimised equivalent of the commented statement above

        if(TimeoutCounter == 1000)
            I2C_Error = true; 

        // Voltage variable now contains the bus voltage in millivolts



        // --------------------------------------- Get the Current Shunt value from the INA219's register -------------------------------

        // Select Shunt Voltage register
        Wire.beginTransmission(I2CADDRESS);                // I2C Address of INA219 in this example is 0x40
        Wire.write((char)(0x01));                          // First byte sent selects the register in the INA219 which we want to read, 01 is shunt voltage
        Wire.endTransmission(true);                        // Send this command over I2C
    
        Wire.requestFrom(I2CADDRESS, 2, true);             // Request 2 bytes from the INA219 and release the bus with a stop
        TimeoutCounter = 0;
        while((Wire.available() < 2) && (TimeoutCounter < 1000))     // wait for 2 bytes back, timeout in approx 1 sec
        {
          TimeoutCounter ++;
          delay(1);
        }   
        
        
        CurrentHi = (Wire.read());			   // Read high byte of the shunt voltage measurement
        CurrentLo = (Wire.read());			   // Read low byte of the shunt voltage measurement

        // If negative current, we force the value to zero to simplify the demo and allow 
        // more space on chart for positive currents. Could for example re-colour chart if negative
        if ((CurrentHi & 0x80) != 0)                                                                  
        {
            Current = 0x0000;                              // Upper bit of result (actually upper 3 bits in 80mV range) reflect the sign
        }
        else
        {
            CurrentHi = CurrentHi & 0x1F;                  // Mask off 3 upper sign bits  SGN SGN SGN S12 S11 S10 S09 S08     where Sxx indicates shunt voltage bit
            CurrentHi = CurrentHi << 8;                    // --- --- --- S12 S11 S10 S09 S08 000 000 000 000 000 000 000 000
                  
            Current = CurrentLo + CurrentHi;               // --- --- --- S12 S11 S10 S09 S08 S07 S06 S05 S04 S03 S02 S01 S00 
            Current = Current * 0.2;                       // Above 13 bits give 8192 possible values
                                                           // In 80mV range of INA219 with 0.05R resistor, 0.08/0.05 = 1.6A full scale
                                                           // Therefore 13-bit value with max value of 8192 represents 1600mA
                                                           // Therefore each bit is 1600/8192 = 0.195 rounded here to 0.2 mA per step
        }
           
           
        if(TimeoutCounter == 1000)
            I2C_Error = true;    
        // Current value now contains the bus current in milliamps
        
        
 
        // #################################################################################################################################
        // Draw the final display
        // #################################################################################################################################
                
        FTImpl.DLStart();                                  // Start the display list. 
        FTImpl.ClearColorRGB(0,0,0);                       // Set the background color to black
        FTImpl.Clear(1,1,1);                               // Clear the screen
      
        FTImpl.Cmd_Append(100000,dloffset_backgnd);        // Append the static background which was previously stored 
                                                           // at address 100000 and has length dloffset_backgnd
          
        // #################################################################################################################################
        // Draw Divisionmarks
        // #################################################################################################################################
      
      
        // Put the values beside the Division marks on the Y scale depending on which range is set 
        if(CurrentRange == RANGE_200)
        {
            FTImpl.Cmd_Append(150000,dloffset_Scale200);   // Append the scale
        }
        else if (CurrentRange == RANGE_800)
        {
            FTImpl.Cmd_Append(160000,dloffset_Scale800);   // Append the scale 
        }
        else  
        {
            FTImpl.Cmd_Append(170000,dloffset_Scale1600);  // Append the scale
        }
      

        // #################################################################################################################################
        // Write Values of Current and Voltage
        // #################################################################################################################################
     
        FTImpl.ColorRGB(0xFF,0xFF,0xFF);//set the color of the point to white
        //FTImpl.Cmd_Number(Voltage_Value_X, Voltage_Value_Y, 14, 0 , Voltage);                         // Display voltage in mV
        FTImpl.Cmd_Number(Current_Value_X, Current_Value_Y, 14, 0, Current);                            // Display current in mA
        FTImpl.Cmd_Number(Voltage_Value_X, Voltage_Value_Y, 14, 0 , Voltage/1000);                      // Display voltage in V (integer part)
        FTImpl.Cmd_Text((Voltage_Value_X+15), Voltage_Value_Y, 14, 0, ".");                             // followed by a dot
        FTImpl.Cmd_Number((Voltage_Value_X+20), Voltage_Value_Y, 14, 0 , int((Voltage%1000)/100));      // and then the fractional part to 1 digit

        // #################################################################################################################################
        // Plot Chart
        // #################################################################################################################################
          
        // First, we add the new measurement taken above to the results buffer             
        ResultBuffer[InputIndex] = Current; 
     
        // Now, we increment the pointer in a circular fashion. 
        // This sets up the pointer for the next reading we take the next time round the loop...
        InputIndex ++;
        if(InputIndex == 300)
        {
            InputIndex = 0;
        }
        // ... and also points to the starting location for the chart drawing 
        OutputIndex = InputIndex;
      
        FTImpl.LineWidth(16);                                      // Set the width
        FTImpl.Begin(FT_LINE_STRIP);                               // A line strip will be used to plot the chart
                    
        ChartIndexX = 0;                                           // Chart will be drawn from offset 0 (left hand side) moving rightwards
    
        while(ChartIndexX < 300)                                   // for each of the 300 chart points...
        {      
            ChartCurrentValue = ResultBuffer[OutputIndex];         // Each pixel is 1mA, can display up to 200mA
       
            // Scale the chart data (currently in mA units with value 0 to 1600) to fit the 200-pixel high chart area    
            // Data may exceed range on the 200mA and 800mA ranges (e.g. if 500mA meausred but on the 200mA range)
            // This is handled when plotting the actual chart below
         
            if (CurrentRange == RANGE_800) 
            {
                ChartCurrentValue = ChartCurrentValue/4;           // Each pixel is 4mA, can display up to 800mA
            }
            else if (CurrentRange == RANGE_1600)
            {
                ChartCurrentValue = ChartCurrentValue/8;           // Each pixel is 8mA, can display up to 1600mA
            }
                   
            FTImpl.ColorRGB(0xC0,0xC0,0xC0);                       // Set a white-gray colour for the chart line
                
            // If the chart value is more than 200 pixels, the line would go off the top of the chart
            // We set a red colour for the line at this point to show that it is off-scale, and the user can change to a 
            // higher range if desired to see the actual value 
            // note: the Current digital meter readout still shows the actual value and is not affetced by the range in any case 
            if(ChartCurrentValue > 200)
            {
                ChartCurrentValue = 200;                           // Force it to 200 so we don't go outside the chart area
                FTImpl.ColorRGB(0xFF,0x00,0x00);                   // Set a red colour for this point to show it's off scale
            }

            // Draw the current point of the line strip
            // The X coordinate is increased from 0 to 300 each time round this loop
            // The Y coordinate is derived from going to the bottom of the chart area and subtracting the value corresponding 
            // to the current value to be displayed (in pixels) 
            FTImpl.Vertex2f(   (ChartTL_X*16 + (ChartIndexX * 16)),   ((ChartBR_Y - (ChartCurrentValue))* 16)    );
        
            // Increment the counter to draw the next point the following time round this loop
            // Because we begin the plot from the location after the last data added, it won't always begin from index 0. 
            // Therefore, must handle roll-over.
            OutputIndex ++;
            if(OutputIndex == 300)
                OutputIndex = 0;

            ChartIndexX ++;  
        }   

        FTImpl.End();             

        // If the readings from INA219 didn't succeed, overlay message to inform user of I2C error
        if(I2C_Error == true)
        {
            FTImpl.ColorRGB(0xFF,0x00,0x00);                          // Red text    
            FTImpl.Cmd_Text((ChartTL_X + (ChartWidth / 2)), (ChartTL_Y + (ChartHeight / 2)), 21, FT_OPT_CENTER, "I2C Communication Error");         
        }

        // #################################################################################################################################
        // Draw Buttons
        // #################################################################################################################################

        FTImpl.ColorRGB(0xFF,0xFF,0xFF);                              // Ensure text is drawn white
	FTImpl.Cmd_FGColor(0x0000FF);                                 // Set the colours for the buttons

        FTImpl.TagMask(1);                                            // Ensure tagging is enabled so that any subsequent items are tagged
 
        // Button for 200mA range
        FTImpl.Tag(2);                                                // Set tag to 2, any subsequent items will be tagged '2'
        if(CurrentRange == RANGE_200)
            FTImpl.Cmd_Button(360,230,30,20,20,FT_OPT_FLAT,"200");    // If 200mA range currently set, draw button with 'pressed in' appearance
        else
            FTImpl.Cmd_Button(360,230,30,20,20,0,"200");              // Otherwise draw it with 'not pressed' 3D appearance
        
        FTImpl.Tag(8);                                                // Set tag to 8, any subsequent items will be tagged '8'
        if(CurrentRange == RANGE_800)                                 
            FTImpl.Cmd_Button(400,230,30,20,20,FT_OPT_FLAT,"800");    // If 800mA range currently set, draw button with 'pressed in' appearance 
        else    
            FTImpl.Cmd_Button(400,230,30,20,20,0,"800");              // Otherwise draw it with 'not pressed' 3D appearance   
  
          FTImpl.Tag(16);                                             // Set tag to 16, any subsequent items will be tagged '16'   
        if(CurrentRange == RANGE_1600)       
            FTImpl.Cmd_Button(440,230,30,20,20,FT_OPT_FLAT,"1600");   // If 1600mA range currently set, draw button with 'pressed in' appearance
        else            
            FTImpl.Cmd_Button(440,230,30,20,20,0,"1600");             // Otherwise draw it with 'not pressed' 3D appearance   

	FTImpl.TagMask(0);                                            // Disable Tag, if drawing other items afterwards, this avoids them being tagged
  
       	FTImpl.DLEnd();                                               // End the display list
	FTImpl.Finish();                                              // Render the display list and wait for the completion of the DL
 
        // #################################################################################################################################
        // Check for touches
        // #################################################################################################################################
 
        ReadWord = FTImpl.Read( REG_TOUCH_TAG);                       // Read the touch tag register to see if 0 (no touch) or if it has one of the 
                                                                      // values corresponding to the tags for the buttons (2, 8 or 16)
        if(ReadWord == 2)
            CurrentRange = RANGE_200;                                 // If '200mA' button touched, set the range variable to show 200mA range 
        if(ReadWord == 8)
            CurrentRange = RANGE_800;                                 // If '800mA' button touched, set the range variable to show 800mA range
        if(ReadWord == 16)
            CurrentRange = RANGE_1600;                                // If '1600mA' button touched, set the range variable to show 1600mA range              
                
        delay(10);                                                    
    }
    


}

void Calibrate()
{ 
    /*************************************************************************/
    /* Below code demonstrates the usage of calibrate function. Calibrate    */
    /* function will wait untill user presses all the three dots. Only way to*/
    /* come out of this api is to reset the coprocessor bit.                 */
    /*************************************************************************/

    /* Construct the display list with grey as background color, informative string "Please Tap on the dot" followed by inbuilt calibration command */
    FTImpl.DLStart();
    FTImpl.ClearColorRGB(64,64,64);
    FTImpl.Clear(1,1,1);    
    FTImpl.ColorRGB(0xff, 0xff, 0xff);
    FTImpl.Cmd_Text((FT_DISPLAYWIDTH/2), (FT_DISPLAYHEIGHT/2), 27, FT_OPT_CENTER, "Please Tap on the dot");
    FTImpl.Cmd_Calibrate(0);
    // Wait for the completion of calibration
    FTImpl.Finish();
}


void setup()
{
    /* Initialize serial print related functionality */
    Serial.begin(9600);

    Wire.begin();
	
    /* Set the Display Enable pin*/   
    //Serial.println("--Start Application--");
    if(BootupConfigure())
    {
        //error case - do not do any thing
    }
    else
    {
        Calibrate();		
        Chart();
    }
    //Serial.println("--End Application--");
}

/* Nothing in loop api */
void loop()
{
}
